@@ -17,6 +17,7 @@ local setmetatable = setmetatable;
 local function dummy_send(sock, data, i, j) return (j-i)+1; end
 
 local _ENV = nil;
+-- luacheck: std none
 
 local async_resolver_methods = {};
 local async_resolver_mt = { __index = async_resolver_methods };
@@ -49,6 +50,11 @@ local function new_async_socket(sock, resolver)
 	if not handler then
 		return nil, err;
 	end
+	if handler.set then
+		-- server_epoll: only watch for incoming data
+		-- avoids sending empty packet on first 'onwritable' event
+		handler:set(true, false);
+	end
 
 	handler.settimeout = function () end
 	handler.setsockname = function (_, ...) return sock:setsockname(...); end
