@@ -72,7 +72,7 @@ function simple_broadcast(kind, node, jids, item, actor, node_obj)
 	end
 
 	local id = new_id();
-	local msg_type = node_obj and node_obj.config.message_type or "headline";
+	local msg_type = node_obj and node_obj.config.notification_type or "headline";
 	local message = st.message({ from = module.host, type = msg_type, id = id })
 		:tag("event", { xmlns = xmlns_pubsub_event })
 			:tag(kind, { node = node })
@@ -112,7 +112,7 @@ function check_node_config(node, actor, new_config) -- luacheck: ignore 212/acto
 end
 
 function is_item_stanza(item)
-	return st.is_stanza(item) and item.attr.xmlns == xmlns_pubsub and item.name == "item";
+	return st.is_stanza(item) and item.attr.xmlns == xmlns_pubsub and item.name == "item" and #item.tags == 1;
 end
 
 module:hook("pubsub-summary/http://www.w3.org/2005/Atom", function (event)
@@ -127,7 +127,7 @@ module:hook("pubsub-summary/http://www.w3.org/2005/Atom", function (event)
 		end
 	end
 	return summary;
-end);
+end, -1);
 
 module:hook("iq/host/"..xmlns_pubsub..":pubsub", handle_pubsub_iq);
 module:hook("iq/host/"..xmlns_pubsub_owner..":pubsub", handle_pubsub_iq);
