@@ -37,6 +37,7 @@ local secure_auth = module:get_option_boolean("s2s_secure_auth", false); -- One
 local secure_domains, insecure_domains =
 	module:get_option_set("s2s_secure_domains", {})._items, module:get_option_set("s2s_insecure_domains", {})._items;
 local require_encryption = module:get_option_boolean("s2s_require_encryption", false);
+local stanza_size_limit = module:get_option_number("s2s_stanza_size_limit", 1024*512);
 
 local measure_connections = module:measure("connections", "amount");
 local measure_ipv6 = module:measure("ipv6", "amount");
@@ -162,7 +163,10 @@ function route_to_new_session(event)
 end
 
 local function keepalive(event)
-	return event.session.sends2s(' ');
+	local session = event.session;
+	if not session.notopen then
+		return event.session.sends2s(' ');
+	end
 end
 
 module:hook("s2s-read-timeout", keepalive, -1);
@@ -272,7 +276,7 @@ function make_authenticated(event)
 end
 
 --- Helper to check that a session peer's certificate is valid
-function check_cert_status(session)
+local function check_cert_status(session)
 	local host = session.direction == "outgoing" and session.to_host or session.from_host
 	local conn = session.conn:socket()
 	local cert
@@ -550,7 +554,7 @@ end
 
 -- Session initialization logic shared by incoming and outgoing
 local function initialize_session(session)
-	local stream = new_xmpp_stream(session, stream_callbacks);
+	local stream = new_xmpp_stream(session, stream_callbacks, stanza_size_limit);
 
 	session.thread = runner(function (stanza)
 		if stanza.name == nil then
@@ -595,8 +599,7 @@ local function initialize_session(session)
 		if data then
 			local ok, err = stream:feed(data);
 			if ok then return; end
-			log("warn", "Received invalid XML: %s", data);
-			log("warn", "Problem was: %s", err);
+			log("debug", "Received invalid XML (%s) %d bytes: %s", tostring(err), #data, data:sub(1, 300):gsub("[\r\n]+", " "):gsub("[%z\1-\31]", "_"));
 			session:close("not-well-formed");
 		end
 	end
